#ifndef _DS18B20_H_
#define _DS18B20_H_

/*
 * ds18b20.h
 *
 * Created: 2011-07-14 12:27:10
 *  Author: Karol
 */ 
/* Author Karol Grzybowski
*
* Obsluga termometru cyfrowego 1-wire DS18B20
*
*/

/* BEFORE USE:
 *
 * define port DS_PORTPINS_S
 * optionally define DS_RESOLUTION */

#include <util/delay.h>
#include <util/crc16.h>
#include "portpins_structs.h"


// global constants
/* read only value, program must wait this time */
/* DS_MAX_CONV_TIME_MS */

/* uses for configuration resolutions */
#define DS_RESOLUTION_9  0
#define DS_RESOLUTION_10 1
#define DS_RESOLUTION_11 2
#define DS_RESOLUTION_12 3

/* check if returned temp is correct */
#define DS_TEMP_OK(t) ((t!=DS_RESET_TEMP) && (t!=DS_CRC_ERROR))


// pin setup
#ifndef DS_PORTPINS_S
#error "struct DS_PORTPINS_S not defined !"
#endif

#ifndef DS_RESOLUTION
#warning "DS_RESOLUTION not defined ! Defaulted to DS_RESOLUTION_12."
#define DS_RESOLUTION DS_RESOLUTION_12
#endif



/* ds maximum conversion time - depends on resolution */
#define DS_MAX_CONV_TIME_MS (ds_tCONV / (1 << (3 - DS_RESOLUTION)))


// time constants
#define  ds_tCONV     750 // ms ;  Temperature Conversion Time
#define  ds_tSLOT      80 // us ;  Time Slot
#define  ds_tREC        5 // us ;  Recovery Time
#define  ds_tLOW0      70 // us ;  Write 0 Low Time
#define  ds_tLOW1       5 // us ;  Write 1 Low Time
#define  ds_tRINT       2 // us ;  Init read
#define  ds_tRDV       10 // us ;  Read Data Valid
#define  ds_tRSTH     500 // us ;  Reset Time High
#define  ds_tRSTL     500 // us ;  Reset Time Low
#define  ds_tPDHIGH    70 // us ;  Presence Detect High
#define  ds_tPDLOW    240 // us ;  Presence Detect Low

#define ds_config_reserved 0x1f
#define ds_config_register (ds_config_reserved | (DS_RESOLUTION << 5))

// global constants
#define  DS_RESET_TEMP 0x0550 // power-on temp
#define  DS_CRC_ERROR  0x1010 // 0x1010 is not correct temp -> 257'C
#define  DS_CRC_POLY   0x18


#define ds_port     PORT(DS_PORTPINS_S)
#define ds_ddr      DDR(DS_PORTPINS_S)
#define ds_pin      PIN(DS_PORTPINS_S)
#define ds_p        P(DS_PORTPINS_S)

#define delay_us(x) _delay_us(x)


// set pin as output
#define ds_out() OUTPUT(DS_PORTPINS_S) //(ds_ddr |=  (1 << ds_p))
#define ds_in()  INPUT(DS_PORTPINS_S)  //(ds_ddr &= ~(1 << ds_p))
#define ds_low() LOW(DS_PORTPINS_S)    // (ds_port &= ~(1 << ds_p))
#define ds_high() HIGH(DS_PORTPINS_S)  // (ds_port |=  (1 << ds_p))


// reset ds1820 and return non-zero if present impulse received, 0 otherwise
static uint8_t ds_reset(void)
{
	// set low on wire
	ds_out();
	ds_low();
	// wait reset time
	delay_us(ds_tRSTL);
	// set high on wire
	ds_high();
	// listen for answer
	ds_in();
	delay_us(ds_tPDHIGH);
	// read answer
	uint8_t signal = (~ds_pin) & (1 << ds_p);
	
	delay_us(ds_tRSTH - ds_tPDHIGH);
	ds_out();
	
	return signal;
}

// write 1 to ds1820
static void ds_write_bit(uint8_t bit)
{
	delay_us(ds_tREC);
	ds_out ();
	ds_low ();
	if (bit)
	{
		delay_us(ds_tLOW1);
		ds_high();
		delay_us(ds_tSLOT - ds_tLOW1);
	}
	else
	{
		delay_us(ds_tLOW0);
		ds_high();
		delay_us(ds_tSLOT - ds_tLOW0);
	}
}
	
// read bit from ds1820 and store in r16 - LSB first
static uint8_t ds_read_bit(uint8_t byte)
{
	uint8_t bit;
	delay_us(ds_tREC);
	ds_out ();
	ds_low ();
	delay_us(ds_tRINT);
	ds_in  ();
	// pull up
	ds_high();
	delay_us(ds_tRDV - ds_tRINT);
	bit = ds_pin;
	byte >>= 1;
	byte |= (bit & (1 << ds_p)) << (7 - ds_p);
	delay_us(ds_tSLOT - (ds_tRDV - ds_tRINT));
	ds_out ();

	return byte;
}	
	
	
// write byte r16 to ds1820
static void ds_write_byte(uint8_t byte)
{
	uint8_t i;
	delay_us(ds_tREC);
	// send 8 bits
	for (i = 0; i < 8; i ++)
	{
		ds_write_bit (byte & 1);
		byte >>= 1;
	}
}
	
// read byte from ds1820 and store it in r16
static uint8_t ds_read_byte(void)
{
	uint8_t i;
	uint8_t byte = 0;
	// read 8 bits
	for (i = 0; i < 8; i++)
	{
		byte = ds_read_bit(byte);
	}	
	return byte;
}

/* config ds18b20 */
/* must be call if DS_RESOLUTION was redefined */
/* return non-zero on success, 0 otherwise */
static uint8_t ds_config(uint8_t t_low, uint8_t t_high)
{
	uint8_t result = ds_reset();
	// skip ROM
	ds_write_byte(0xCC);
	// convert T
	ds_write_byte(0x4E);
	// write config
	ds_write_byte(t_high);
	ds_write_byte(t_low);
	ds_write_byte(ds_config_register);
	
	return result;
}

/* ask for temp conversion */
/* must be call DS_MAX_CONV_TIME_MS before ds_read */
/* return non-zero on success, 0 otherwise */
static uint8_t ds_ask_temp(void)
{
	uint8_t result = ds_reset();
	// skip ROM
	ds_write_byte(0xCC);
	// convert T
	ds_write_byte(0x44);
	
	return result;
};

// read temp
static int16_t ds_read_temp(void)
{
	int16_t temp;
	
	ds_reset();
	// skip ROM
	ds_write_byte(0xCC);
	// read scratch pad
	ds_write_byte(0xBE);
	
	// LSB of temp
	temp = ds_read_byte();
	// MSB of temp
	temp |= ds_read_byte() << 8;
	
	return temp;
}

// read temp and check crc sum
static int16_t ds_read_temp_crc()
{
	int16_t temp;
	uint8_t i, data, crc;
	
	if (!ds_reset())
	{
		return DS_RESET_TEMP;
	}
	// skip ROM
	ds_write_byte(0xCC);
	// read scratch pad
	ds_write_byte(0xBE);
	
	// LSB of temp
	data = ds_read_byte();
	temp = (int16_t)data;
	crc = _crc_ibutton_update(0, data);
	// MSB of temp
	data = ds_read_byte();
	temp |= (int16_t)(data) << 8;
	crc = _crc_ibutton_update(crc, data);
	
	for (i = 0; i < 7; i++)
	{
		data = ds_read_byte();
		crc = _crc_ibutton_update(crc, data);
	}
	
	if (crc)
	{
		return DS_CRC_ERROR;
	}
	
	return temp;
}

/*
static int16_t ds_read_temp_crc()
{
	int16_t temp;
	uint8_t i, data, crc;
	
	if (!ds_reset())
 2e2:	0e 94 82 00 	call	0x104	; 0x104 <ds_reset>
 2e6:	88 23       	and	r24, r24
 2e8:	09 f4       	brne	.+2      	; 0x2ec <main+0x118>
	{
		return DS_RESET_TEMP;
	}
 2ea:	7b c0       	rjmp	.+246    	; 0x3e2 <main+0x20e>
 	// skip ROM
	ds_write_byte(0xCC);
 2ec:	8c ec       	ldi	r24, 0xCC	; 204
 2ee:	0e 94 9e 00 	call	0x13c	; 0x13c <ds_write_byte>
	// read scratch pad
	ds_write_byte(0xBE);
 2f2:	8e eb       	ldi	r24, 0xBE	; 190
 2f4:	0e 94 9e 00 	call	0x13c	; 0x13c <ds_write_byte>
	// LSB of temp
	data = ds_read_byte();
	temp = (int16_t)data;
 2f8:	0e 94 c6 00 	call	0x18c	; 0x18c <ds_read_byte>
 2fc:	08 2f       	mov	r16, r24 ; LSB stored in r16
 2fe:	80 e0       	ldi	r24, 0x00	; 0
 	crc = _crc_ibutton_update(0, data);
 300:	60 2f       	mov	r22, r16
 302:	0e 94 41 00 	call	0x82	; 0x82 <_crc_ibutton_update>
 306:	18 2f       	mov	r17, r24
 	// MSB of temp
	data = ds_read_byte();
	temp |= (int16_t)(data) << 8;
 308:	0e 94 c6 00 	call	0x18c	; 0x18c <ds_read_byte>
 30c:	e8 2e       	mov	r14, r24 ; MSB stored in r14
 30e:	81 2f       	mov	r24, r17
 	crc = _crc_ibutton_update(crc, data);
 310:	6e 2d       	mov	r22, r14
 312:	0e 94 41 00 	call	0x82	; 0x82 <_crc_ibutton_update>
 316:	d8 2e       	mov	r13, r24
 	for (i = 0; i < 7; i++)
	{
		data = ds_read_byte();
		crc = _crc_ibutton_update(crc, data);
	}
 318:	17 e0       	ldi	r17, 0x07	; 7
 31a:	0e 94 c6 00 	call	0x18c	; 0x18c <ds_read_byte>
 31e:	68 2f       	mov	r22, r24
 320:	8d 2d       	mov	r24, r13
 322:	0e 94 41 00 	call	0x82	; 0x82 <_crc_ibutton_update>
 326:	d8 2e       	mov	r13, r24
 328:	11 50       	subi	r17, 0x01	; 1
 32a:	b9 f7       	brne	.-18     	; 0x31a <main+0x146>
 	if (crc)
	{
		return DS_CRC_ERROR;
	}
 32c:	88 23       	and	r24, r24
 32e:	09 f0       	breq	.+2      	; 0x332 <main+0x15e>
 330:	5b c0       	rjmp	.+182    	; 0x3e8 <main+0x214>
 332:	a0 e0       	ldi	r26, 0x00	; r27:r26 contains: 0x?? 00
 334:	ed 01       	movw	r28, r26	; r29:r28 contains: 0x?? 00
 336:	10 e0       	ldi	r17, 0x00	; r17:r16 contains: 0x00 LSB
 	return temp;
338:	0c 2b       	or	r16, r28	; r17:r16 := r17:r16 OR r29:r28, where r29 is undefined instead of MSB
 33a:	1d 2b       	or	r17, r29 // temp in r17:r29

 
 33c:	b5 e0       	ldi	r27, 0x05	; 5
 33e:	00 35       	cpi	r16, 0x50	; 80
 340:	1b 07       	cpc	r17, r27
 342:	09 f4       	brne	.+2      	; 0x346 <main+0x172>
 344:	53 c0       	rjmp	.+166    	; 0x3ec <main+0x218>
 346:	e0 e1       	ldi	r30, 0x10	; 16
 348:	00 31       	cpi	r16, 0x10	; 16
 34a:	1e 07       	cpc	r17, r30
 34c:	09 f4       	brne	.+2      	; 0x350 <main+0x17c>
 34e:	4e c0       	rjmp	.+156    	; 0x3ec <main+0x218>
 
 */

#endif /* _DS18B20_H_ */
